library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity alignator is
	generic ( maxraws : integer := 352*288/64*(16+8+8));
	port (
		clk	: in	std_logic;
		reset	: in	std_logic;
		din	: in	std_logic_vector(7 downto 0);
		dine	: in	std_logic;
		cidle	: in	std_logic;
		rawe	: in	std_logic;
		dout	: out	std_logic_vector(7 downto 0);
		doute	: out	std_logic
		);
end alignator;

architecture arh of alignator is

	signal incnt : unsigned(1 downto 0);
	signal allraws, cdone, dummye : std_logic;

	type state_t is (wait_done, dummy3, dummy2, dummy1, dummy_done, waitstate1, waitstate2, waitstate3);
	signal state: state_t;

	signal nraws : integer range 0 to 352*288-1;

begin

	allraws <= '1' when nraws >= maxraws else '0';
	cdone <= cidle and allraws;
	dummye <= '1' when state=dummy3 or state=dummy2 or state=dummy1 else '0';

	doute <= dine or dummye;
	dout <= din when dine='1' else x"FF";

	counter0: process (clk)
	begin
		if rising_edge(clk) then
			if rawe = '1' then
				nraws <= nraws + 1;
			end if;

			if reset = '1' then
				nraws <= 0;
			end if;
		end if;
	end process;

	counter1: process (clk)
	begin
		if rising_edge(clk) then
			if dine = '1' then
				incnt <= (incnt + 1) mod 4;
			end if;

			if reset = '1' then
				incnt <= (others => '0');
			end if;
		end if;
	end process;

	process (clk)
	begin
		if rising_edge(clk) then
			case state is
				when wait_done =>
					if cdone = '1' then
						state <= waitstate1;
					end if;

				when waitstate1 => state <= waitstate2;
				when waitstate2 => state <= waitstate3;

				when waitstate3 =>
					case incnt(1 downto 0) is
						when "01" => state <= dummy3;
						when "10" => state <= dummy2;
						when "11" => state <= dummy1;
						when others => state <= dummy_done;
					end case;

				when dummy3 => state <= dummy2;
				when dummy2 => state <= dummy1;
				when dummy1 => state <= dummy_done;

				when others =>	-- dummy_done
					state <= dummy_done;
			end case;

			if reset = '1' then
				state <= wait_done;
			end if;
		end if;
	end process;

end arh;
