library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity compressor is
	port (	clk	: in	std_logic;
		reset	: in	std_logic;
		din	: in	std_logic_vector(7 downto 0);
		dine	: in	std_logic;
		dout	: out	std_logic_vector(7 downto 0);
		doute	: out	std_logic;
		idle	: out	std_logic
		);
end compressor;

architecture arh of compressor is

	-- fifo iface
	signal ffd : std_logic_vector(7 downto 0);
	signal ffdrq, ffempty : std_logic;

	signal maxzeros, iszero : boolean;
	signal nzeros : unsigned(7 downto 0);
	signal nzeros_reset, nzeros_inc : std_logic;

	type state_t is (getting_data, counting_zeros);
	signal state, next_state : state_t;

	signal statec : integer;

begin

	statec <= 0 when state = getting_data else 1;


	idle <= ffempty;
	iszero <= true when ffd = x"00" else false;
	maxzeros <= true when nzeros = x"FE" else false;


	fifo0: entity work.fifo
		generic map (len => 64)
		port map (clk, reset, din, dine, ffd, ffdrq, ffempty);


	nzeros_counter: process (clk)
		function inc(i : unsigned) return unsigned is
			variable ii : integer;
		begin
			ii := (to_integer(i) + 1) mod (2**i'length);
			return to_unsigned(ii, i'length);
		end function;
	begin
		if rising_edge(clk) then
			if nzeros_inc = '1' then
				nzeros <= inc(nzeros);
			end if;

			if nzeros_reset = '1' then
				nzeros <= x"00";
			end if;
		end if;
	end process;

	process (state, ffempty, ffd, maxzeros, iszero, nzeros)
	begin
		doute <= '0';
		dout <= (others => 'X');
		nzeros_reset <= '0';
		nzeros_inc <= '0';
		next_state <= state;
		ffdrq <= '0';

		if ffempty = '0' then
			case state is
				when getting_data =>
					doute <= '1';
					dout <= ffd;
					ffdrq <= '1';
					if iszero then
						next_state <= counting_zeros;
						nzeros_reset <= '1';
					end if;

				when counting_zeros =>
					if iszero then
						ffdrq <= '1';
						nzeros_inc <= '1';
						if maxzeros then
							doute <= '1';
							dout <= x"FF";
							next_state <= getting_data;
						end if;
					else
						doute <= '1';	-- izbaci koliko je skupio 0
						dout <= std_logic_vector(nzeros);
						next_state <= getting_data;
					end if;

				when others =>
					next_state <= getting_data;
			end case;
		end if;
	end process;

	process (clk)
	begin
		if rising_edge(clk) then
			state <= next_state;

			if reset = '1' then
				state <= getting_data;
			end if;
		end if;
	end process;

end arh;
