library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity dct2d is
	port (	clk	: in	std_logic;
		reset	: in	std_logic;
		din	: in	std_logic_vector(7 downto 0);
		dine	: in	std_logic;
		dout	: out	signed(7 downto 0);
		doute	: out	std_logic;
		busy	: out	std_logic
		);
end dct2d;

architecture arh of dct2d is

	type state_t is (reading_rows, wait_dct1, prepare_cols, calculating_cols);

	type coeffs8 is array(0 to 7) of signed(7 downto 0);
	signal dctin, dctout, rowregs, colout : coeffs8;
	signal rowvector, colvector : std_logic_vector(0 to 8*8-1);

	signal state : state_t;
	signal ibusy : std_logic := '0';

	signal row_we, zz_we : std_logic;
	signal coefid, bufaddrin : integer range 0 to 7;

--	signal statec : integer;

begin

	busy <= ibusy;

--	with state select
--		statec <= 0 when reading_rows,
--		          1 when wait_dct1,
--			  2 when prepare_cols,
--			  3 when calculating_cols,
--			  -1 when others;

	zzbuf: entity work.zzbuf
		generic map (width => 8)
		port map (
			clk,
			reset,
			zz_we,
			rowvector,
			dout,
			doute
		);

	rowvectorconcator: for i in 0 to 7 generate
		rowvector(i*8 to i*8+7) <= std_logic_vector(dctout(i));
	end generate;
	colvectorextractor: for i in 0 to 7 generate
		colout(i) <= signed(colvector(i*8 to i*8+8-1));
	end generate;

	dctbuf: entity work.dct2dbuf
		generic map (width => 8)
		port map (
			clk,
			bufaddrin, bufaddrin,
			row_we,
			rowvector, colvector
		);

	-- odabir sto ce se kada dodijeliti DCT-u (rowregs ili colout, odnosno, po retcima ili po stupcima)
	dctin <= rowregs when state = reading_rows or state = wait_dct1 else colout;

	dct0: entity work.dct81d
		generic map (width => 8)
		port map (
			clk,
			dctin(0),  dctin(1),  dctin(2),  dctin(3),  dctin(4),  dctin(5),  dctin(6),  dctin(7),
			dctout(0), dctout(1), dctout(2), dctout(3), dctout(4), dctout(5), dctout(6), dctout(7)
		);

	--ibusy <= '0' when state = reading_rows else '1';

	process (clk) is
	begin
		if rising_edge(clk) then
			row_we <= '0';			-- zapis DCT-iziranog retka
			zz_we <= '0';			-- zapis 2D-DCT-iziranog stupca

			if reset = '1' then
				bufaddrin <= 7;		-- adresa retka u prvom prolazu (prvo uveca)
				coefid <= 0;
				state <= reading_rows;
				ibusy <= '0';
			else
				case state is
					when reading_rows =>	-- ucitava podatke i odmah racuna dct po redovima
						if dine = '1' then
							rowregs(coefid) <= signed(din) xor x"80";
							if coefid = 3 then
								row_we <= '1';	-- gotov h dct
							elsif coefid = 7 then
								if bufaddrin = 6 then
									state <= wait_dct1;
								end if;
								if bufaddrin = 7 then
									bufaddrin <= 0;
								else
									bufaddrin <= bufaddrin + 1;
								end if;
							end if;

							if coefid = 7 then
								coefid <= 0;
							else
								coefid <= coefid + 1;
							end if;

							if coefid = 6 and bufaddrin = 6 then
								ibusy <= '1';
							end if;
						end if;

					when wait_dct1 =>	-- ceka zadnji dct
						bufaddrin <= 7;
						if coefid = 3 then
							row_we <= '1';	-- gotov zadnji h dct
						elsif coefid = 4 then
							state <= prepare_cols;
							bufaddrin <= 0;
						end if;
						coefid <= coefid + 1;

					when prepare_cols =>
						bufaddrin <= bufaddrin + 1;
						if bufaddrin = 3 then
							zz_we <= '1';
							state <= calculating_cols;
						end if;

					when calculating_cols =>	-- izracunava dct po stupcima
						zz_we <= '1';
						if bufaddrin = 7 then
							bufaddrin <= 0;
						else
							bufaddrin <= bufaddrin + 1;
						end if;
						if bufaddrin = 2 then
							ibusy <= '0';
						elsif bufaddrin = 3 then
							bufaddrin <= 7;
							coefid <= 0;
							state <= reading_rows;
							zz_we <= '0';
						end if;

					when others => null;
				end case;
			end if;
		end if;
	end process;

end arh;
