library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity dct81d is
	generic ( width : integer);
	port (	clk : in std_logic;
		din0 : in signed(width-1 downto 0);
		din1 : in signed(width-1 downto 0);
		din2 : in signed(width-1 downto 0);
		din3 : in signed(width-1 downto 0);
		din4 : in signed(width-1 downto 0);
		din5 : in signed(width-1 downto 0);
		din6 : in signed(width-1 downto 0);
		din7 : in signed(width-1 downto 0);

		dout0 : out signed(width-1 downto 0);
		dout1 : out signed(width-1 downto 0);
		dout2 : out signed(width-1 downto 0);
		dout3 : out signed(width-1 downto 0);
		dout4 : out signed(width-1 downto 0);
		dout5 : out signed(width-1 downto 0);
		dout6 : out signed(width-1 downto 0);
		dout7 : out signed(width-1 downto 0)
		);
end dct81d;

architecture arh of dct81d is

	constant sqrt2 : real := 1.41421;

	type coefs8 is array(0 to 7) of signed(width+3-1 downto 0);
	signal stage1, stage2, stage3 : coefs8;

begin

	process (clk) is

		function prosiri(a: signed)
		return signed is
		begin
			return to_signed(to_integer(to_01(a)), width+3);
		end function;

		function trim(a : signed)
		return signed is
			variable tmp : signed(width-1 downto 0);
		begin
			tmp := a(a'high downto a'low+3);
			if a(a'low+2) = '1' then
				tmp := tmp + 1;
			end if;
			return tmp;
		end function;

		function fraction(f : real; size : integer) 
		return signed is
			variable ifract : integer;
		begin
			ifract := integer(real(2**(size-2)-1) * f);
			return to_signed(ifract, size);
		end function;

		function fractmult(a : signed; b : real)
		return signed is
			constant width : integer := a'length;
			constant dwidth : integer := width*2;
			variable tmp : signed(dwidth-1 downto 0);
		begin
			tmp := a * fraction(b, width);
			return tmp(dwidth-3 downto width-2);
		end function;

		function rotator1(i0, i1 : signed; hl : integer)
		return signed is
			variable a, aa, b, bb : signed(i0'range);
		begin
			if hl = 0 then
				a := i0;
				b := i1;
			else
				a := i1;
				b := -i0;
			end if;
			aa := fractmult(a, 0.9808);
			--aa := a - (a/16);		-- 0.9808 ~= 0.9375
			--bb := fractmult(b, 0.1951);
			bb := (b/8) + (b/16);		-- 0.1951 ~= 0.1875
			return aa + bb;
		end function;

		function rotator3(i0, i1 : signed; hl : integer)
		return signed is
			variable a, aa, b, bb : signed(i0'range);
		begin
			if hl = 0 then
				a := i0;
				b := i1;
			else
				a := i1;
				b := -i0;
			end if;
			aa := fractmult(a, 0.8315);
			--bb := fractmult(b, 0.5556);
			bb := (b/2) + (b/16);		-- 0.5556 ~= 0.5625
			return aa + bb;
		end function;

		function rotator6(i0, i1 : signed; hl : integer)
		return signed is
			variable a, aa, b, bb : signed(i0'range);
		begin
			if hl = 0 then
				a := i0;
				b := i1;
			else
				a := i1;
				b := -i0;
			end if;
			aa := fractmult(a, 0.5412);
			bb := fractmult(b, 1.3066);
			return aa + bb;
		end function;

		--function mulsqrt2(a : signed)
		--return signed is
		--begin
		--	return a + (a/4) + (a/8) + (a/16);
		--end function;

	begin
		if rising_edge(clk) then
			-- stage 1
			stage1(0) <= prosiri(din0) + prosiri(din7);
			stage1(1) <= prosiri(din1) + prosiri(din6);
			stage1(2) <= prosiri(din2) + prosiri(din5);
			stage1(3) <= prosiri(din3) + prosiri(din4);
			stage1(4) <= prosiri(din3) - prosiri(din4);
			stage1(5) <= prosiri(din2) - prosiri(din5);
			stage1(6) <= prosiri(din1) - prosiri(din6);
			stage1(7) <= prosiri(din0) - prosiri(din7);

			-- stage 2
			stage2(0) <= stage1(0) + stage1(3);
			stage2(1) <= stage1(1) + stage1(2);
			stage2(2) <= stage1(1) - stage1(2);
			stage2(3) <= stage1(0) - stage1(3);
			stage2(4) <= rotator3(stage1(4), stage1(7), 0);
			stage2(5) <= rotator1(stage1(5), stage1(6), 0);
			stage2(6) <= rotator1(stage1(5), stage1(6), 1);
			stage2(7) <= rotator3(stage1(4), stage1(7), 1);

			-- stage 3
			stage3(0) <= stage2(0) + stage2(1);
			stage3(1) <= stage2(0) - stage2(1);
			stage3(2) <= rotator6(stage2(2), stage2(3), 0);
			stage3(3) <= rotator6(stage2(2), stage2(3), 1);
			stage3(4) <= stage2(4) + stage2(6);
			stage3(5) <= stage2(7) - stage2(5);
			stage3(6) <= stage2(4) - stage2(6);
			stage3(7) <= stage2(7) + stage2(5);

			-- stage 4
			dout0 <= trim(stage3(0));
			dout4 <= trim(stage3(1));
			dout2 <= trim(stage3(2));
			dout6 <= trim(stage3(3));
			dout7 <= trim(stage3(7) - stage3(4));
			dout3 <= trim(fractmult(stage3(5), sqrt2));	-- *sqrt(2)
			dout5 <= trim(fractmult(stage3(6), sqrt2));	-- *sqrt(2)
			dout1 <= trim(stage3(7) + stage3(4));
		end if;
	end process;

end arh;
