library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity fifo is
	generic (
		len : integer := 64;
		width : integer := 8
		);
	port (
		clk	: in	std_logic;
		reset	: in	std_logic;
		din	: in	std_logic_vector(width-1 downto 0);
		dine	: in	std_logic;
		dout	: out	std_logic_vector(width-1 downto 0);
		doutrq	: in	std_logic;
		empty	: out	std_logic
		);
end fifo;

architecture arh of fifo is

	type fifo_dt is array(0 to len-1) of std_logic_vector(width-1 downto 0);
	signal fifo : fifo_dt;
	signal fi, fo : integer range 0 to len-1;

begin

	empty <= '1' when fi = fo else '0';
	dout <= fifo(fo);

	process (clk)
	begin
		if rising_edge(clk) then
			if dine = '1' then
				fifo(fi) <= din;
				fi <= (fi + 1) mod len;
			end if;

			if doutrq = '1' then
				fo <= (fo + 1) mod len;
			end if;

			if reset = '1' then
				fi <= 0;
				fo <= 0;
			end if;
		end if;
	end process;

end arh;
