library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use std.textio.all;
use work.txt_util.all;

entity ppmload is
	generic (filename : string);
	port (
		clk	: out	std_logic;
		yo	: out	std_logic_vector(7 downto 0);
		uvo	: out	std_logic_vector(7 downto 0);
		href	: out	std_logic;
		vsync	: out	std_logic;
		pic_done : out	boolean
	);
end ppmload;

architecture arh of ppmload is

	constant W : integer := 352;
	constant H : integer := 288;

	signal iclk : std_logic := '0';
	signal done : boolean := false;

begin

	pic_done <= done;
	clk <= iclk;

	reader: process
		file f: text open read_mode is filename;
		variable l: line;
		variable ny, nu, nv, x, y: integer := 0;
		variable unv : boolean := true;
	begin
		href <= '0';
		vsync <= '0';
		yo <= (others => 'U');
		uvo <= (others => 'U');

		readline(f, l);	-- header
		readline(f, l);	-- komentar
		readline(f, l); -- dimenzije
		readline(f, l); -- boje
		ml: while not endfile(f) loop
			wait until rising_edge(iclk);
			wait until falling_edge(iclk);
			vsync <= '1';
			for i in 0 to 1 loop
				wait until falling_edge(iclk);
			end loop;
			vsync <= '0';
			wait until falling_edge(iclk);

			for y in 0 to H-1 loop
				href <= '1';
				for x in 0 to W-1 loop
					readline(f, l);
					read(l, ny);
					readline(f, l);
					read(l, nu);
					readline(f, l);
					read(l, nv);
					yo <= std_logic_vector(to_unsigned(ny, yo'length));
					if unv then
						uvo <= std_logic_vector(to_unsigned(nu, uvo'length));
					else
						uvo <= std_logic_vector(to_unsigned(nv, uvo'length));
					end if;
					unv := not unv;
					wait until falling_edge(iclk);
					yo <= (others => 'U');
					uvo <= (others => 'U');
				end loop;
				href <= '0';

				wait until falling_edge(iclk);

				--exit ml when y = 16; --- early done
			end loop;

			exit ml;
		end loop;
		done <= true;
		print("procitana slika");
		wait;
	end process;

	clkgen: process
	begin
		while not done loop
			iclk <= not iclk;
			wait for 50 ns;
		end loop;
		for i in 0 to W*16 loop
			iclk <= not iclk;
			wait for 50 ns;
		end loop;
		wait;
	end process;

end arh;
