library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity quantizer is
	port (	clk	: in	std_logic;
		reset	: in	std_logic;
		dine	: in	std_logic;
		din	: in	signed(7 downto 0);
		doute	: out	std_logic;
		dout	: out	std_logic_vector(7 downto 0)
		);
end quantizer;

architecture arh of quantizer is

	signal collectbuf : std_logic_vector(0 to 7);
	signal coefid : integer range 0 to 63;

	signal roundw : unsigned(2 downto 0);
	signal rounder : signed(7 downto 0);
	signal rounded : std_logic_vector(7 downto 0);

begin

	decoder: process (roundw, din)
	begin
		rounder <= (others => '0');
		rounder(to_integer(roundw)) <= din(to_integer(roundw));
	end process;

	rounded <= std_logic_vector(din + rounder);

	quantizator: process (clk)
	begin
		if rising_edge(clk) then
			if reset = '1' then
				coefid <= 0;
			elsif dine = '1' then
				dout <= (others => 'X');
				doute <= '0';
				case coefid is
					when 0 =>
						collectbuf(0 to 3) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 1 =>
						doute <= '1';
						dout <= collectbuf(0 to 3) & rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 2 =>
						collectbuf(0 to 4) <= rounded(7 downto 3);
						roundw <= o"2";	-- w=5
					when 3 =>
						doute <= '1';
						dout <= collectbuf(0 to 4) & rounded(7 downto 5);
						collectbuf(0 to 0) <= rounded(4 downto 4);
						roundw <= o"3";	-- w=4
					when 4 =>
						collectbuf(1 to 4) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 5 =>
						doute <= '1';
						dout <= collectbuf(0 to 4) & rounded(7 downto 5);
						collectbuf(0 to 0) <= rounded(4 downto 4);
						roundw <= o"3";	-- w=4
					when 6 =>
						collectbuf(1 to 4) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 7 =>
						doute <= '1';
						dout <= collectbuf(0 to 4) & rounded(7 downto 5);
						collectbuf(0 to 0) <= rounded(4 downto 4);
						roundw <= o"3";	-- w=4
					when 8 =>
						collectbuf(1 to 4) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 9 =>
						doute <= '1';
						dout <= collectbuf(0 to 4) & rounded(7 downto 5);
						collectbuf(0 to 0) <= rounded(4 downto 4);
						roundw <= o"3";	-- w=4
					when 10 =>
						collectbuf(1 to 3) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 11 =>
						doute <= '1';
						dout <= collectbuf(0 to 3) & rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 12 =>
						collectbuf(0 to 3) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 13 =>
						doute <= '1';
						dout <= collectbuf(0 to 3) & rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 14 =>
						collectbuf(0 to 2) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 15 =>
						collectbuf(3 to 5) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 16 =>
						doute <= '1';
						dout <= collectbuf(0 to 5) & rounded(7 downto 6);
						collectbuf(0 to 0) <= rounded(5 downto 5);
						roundw <= o"4";	-- w=3
					when 17 =>
						collectbuf(1 to 3) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 18 =>
						collectbuf(4 to 6) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 19 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
						collectbuf(0 to 1) <= rounded(6 downto 5);
						roundw <= o"4";	-- w=3
					when 20 =>
						collectbuf(2 to 4) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 21 =>
						collectbuf(5 to 6) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 22 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
						collectbuf(0 to 0) <= rounded(6 downto 6);
						roundw <= o"5";	-- w=2
					when 23 =>
						collectbuf(1 to 3) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 24 =>
						collectbuf(4 to 6) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 25 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
						collectbuf(0 to 1) <= rounded(6 downto 5);
						roundw <= o"4";	-- w=3
					when 26 =>
						collectbuf(2 to 4) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 27 =>
						collectbuf(5 to 6) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 28 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 29 =>
						collectbuf(0 to 1) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 30 =>
						collectbuf(2 to 3) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 31 =>
						collectbuf(4 to 5) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 32 =>
						doute <= '1';
						dout <= collectbuf(0 to 5) & rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 33 =>
						collectbuf(0 to 1) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 34 =>
						collectbuf(2 to 3) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 35 =>
						collectbuf(4 to 4) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 36 =>
						collectbuf(5 to 5) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 37 =>
						collectbuf(6 to 6) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 38 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
						collectbuf(0 to 0) <= rounded(6 downto 6);
						roundw <= o"5";	-- w=2
					when 39 =>
						collectbuf(1 to 2) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 40 =>
						collectbuf(3 to 4) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 41 =>
						collectbuf(5 to 5) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 42 =>
						collectbuf(6 to 6) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 43 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 44 =>
						collectbuf(0 to 0) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 45 =>
						collectbuf(1 to 1) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 46 =>
						collectbuf(2 to 2) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 47 =>
						collectbuf(3 to 3) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 48 =>
						collectbuf(4 to 4) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 49 =>
						collectbuf(5 to 5) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 50 =>
						collectbuf(6 to 6) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 51 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when others => null;
				end case;
				if coefid = 63 then
					coefid <= 0;
				else
					coefid <= coefid + 1;
				end if;
			end if;
		end if;
	end process;

end arh;
