library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity quantizer is
	port (	clk	: in	std_logic;
		reset	: in	std_logic;
		dine	: in	std_logic;
		din	: in	signed(7 downto 0);
		doute	: out	std_logic;
		dout	: out	std_logic_vector(7 downto 0)
		);
end quantizer;

architecture arh of quantizer is

	signal collectbuf : std_logic_vector(0 to 127);
	signal coefid : integer range 0 to 63;

	signal roundw : unsigned(2 downto 0);
	signal rounder : signed(7 downto 0);
	signal rounded : std_logic_vector(7 downto 0);

begin

	decoder: process (roundw, din)
	begin
		rounder <= (others => '0');
		rounder(to_integer(roundw)) <= din(to_integer(roundw));
	end process;

	rounded <= std_logic_vector(din + rounder);

	quantizator: process (clk)
	begin
		if rising_edge(clk) then
			if reset = '1' then
				coefid <= 0;
			elsif dine = '1' then
				dout <= (others => 'X');
				doute <= '0';

				case coefid is
					when 0 =>
						collectbuf(  0 to   3) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 1 =>
						doute <= '1';
						dout <= collectbuf(  0 to   3) & rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 2 =>
						collectbuf(  4 to   8) <= rounded(7 downto 3);
						roundw <= o"2";	-- w=5
					when 3 =>
						doute <= '1';
						dout <= collectbuf(  4 to   8) & rounded(7 downto 5);
						collectbuf(  9 to   9) <= rounded(4 downto 4);
						roundw <= o"3";	-- w=4
					when 4 =>
						collectbuf( 10 to  13) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 5 =>
						doute <= '1';
						dout <= collectbuf(  9 to  13) & rounded(7 downto 5);
						collectbuf( 14 to  14) <= rounded(4 downto 4);
						roundw <= o"3";	-- w=4
					when 6 =>
						collectbuf( 15 to  18) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 7 =>
						doute <= '1';
						dout <= collectbuf( 14 to  18) & rounded(7 downto 5);
						collectbuf( 19 to  19) <= rounded(4 downto 4);
						roundw <= o"3";	-- w=4
					when 8 =>
						collectbuf( 20 to  23) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 9 =>
						doute <= '1';
						dout <= collectbuf( 19 to  23) & rounded(7 downto 5);
						collectbuf( 24 to  24) <= rounded(4 downto 4);
						roundw <= o"3";	-- w=4
					when 10 =>
						collectbuf( 25 to  27) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 11 =>
						doute <= '1';
						dout <= collectbuf( 24 to  27) & rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 12 =>
						collectbuf( 28 to  31) <= rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 13 =>
						doute <= '1';
						dout <= collectbuf( 28 to  31) & rounded(7 downto 4);
						roundw <= o"3";	-- w=4
					when 14 =>
						collectbuf( 32 to  34) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 15 =>
						collectbuf( 35 to  37) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 16 =>
						doute <= '1';
						dout <= collectbuf( 32 to  37) & rounded(7 downto 6);
						collectbuf( 38 to  38) <= rounded(5 downto 5);
						roundw <= o"4";	-- w=3
					when 17 =>
						collectbuf( 39 to  41) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 18 =>
						collectbuf( 42 to  44) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 19 =>
						doute <= '1';
						dout <= collectbuf( 38 to  44) & rounded(7 downto 7);
						collectbuf( 45 to  46) <= rounded(6 downto 5);
						roundw <= o"4";	-- w=3
					when 20 =>
						collectbuf( 47 to  49) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 21 =>
						collectbuf( 50 to  51) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 22 =>
						doute <= '1';
						dout <= collectbuf( 45 to  51) & rounded(7 downto 7);
						collectbuf( 52 to  52) <= rounded(6 downto 6);
						roundw <= o"5";	-- w=2
					when 23 =>
						collectbuf( 53 to  55) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 24 =>
						collectbuf( 56 to  58) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 25 =>
						doute <= '1';
						dout <= collectbuf( 52 to  58) & rounded(7 downto 7);
						collectbuf( 59 to  60) <= rounded(6 downto 5);
						roundw <= o"4";	-- w=3
					when 26 =>
						collectbuf( 61 to  63) <= rounded(7 downto 5);
						roundw <= o"4";	-- w=3
					when 27 =>
						collectbuf( 64 to  65) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 28 =>
						doute <= '1';
						dout <= collectbuf( 59 to  65) & rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 29 =>
						collectbuf( 66 to  67) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 30 =>
						collectbuf( 68 to  69) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 31 =>
						collectbuf( 70 to  71) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 32 =>
						doute <= '1';
						dout <= collectbuf( 66 to  71) & rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 33 =>
						collectbuf( 72 to  73) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 34 =>
						collectbuf( 74 to  75) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 35 =>
						collectbuf( 76 to  76) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 36 =>
						collectbuf( 77 to  77) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 37 =>
						collectbuf( 78 to  78) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 38 =>
						doute <= '1';
						dout <= collectbuf( 72 to  78) & rounded(7 downto 7);
						collectbuf( 79 to  79) <= rounded(6 downto 6);
						roundw <= o"5";	-- w=2
					when 39 =>
						collectbuf( 80 to  81) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 40 =>
						collectbuf( 82 to  83) <= rounded(7 downto 6);
						roundw <= o"5";	-- w=2
					when 41 =>
						collectbuf( 84 to  84) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 42 =>
						collectbuf( 85 to  85) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 43 =>
						doute <= '1';
						dout <= collectbuf( 79 to  85) & rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 44 =>
						collectbuf( 86 to  86) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 45 =>
						collectbuf( 87 to  87) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 46 =>
						collectbuf( 88 to  88) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 47 =>
						collectbuf( 89 to  89) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 48 =>
						collectbuf( 90 to  90) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 49 =>
						collectbuf( 91 to  91) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 50 =>
						collectbuf( 92 to  92) <= rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when 51 =>
						doute <= '1';
						dout <= collectbuf( 86 to  92) & rounded(7 downto 7);
						roundw <= o"6";	-- w=1
					when others => null;
				end case;

				if coefid = 63 then
					coefid <= 0;
				else
					coefid <= coefid + 1;
				end if;
			end if;
		end if;
	end process;

end arh;
