library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity quantizer is
	port (	clk	: in	std_logic;
		reset	: in	std_logic;
		dine	: in	std_logic;
		din	: in	signed(7 downto 0);
		doute	: out	std_logic;
		dout	: out	std_logic_vector(7 downto 0)
		);
end quantizer;

architecture arh of quantizer is

	signal collectbuf : std_logic_vector(0 to 7);
	signal coefid : integer range 0 to 63;

	signal roundw : unsigned(2 downto 0);
	signal rounder : signed(7 downto 0);
	signal rounded : std_logic_vector(7 downto 0);

begin

	decoder: process (roundw, din)
	begin
		rounder <= (others => '0');
		rounder(to_integer(roundw)) <= din(to_integer(roundw));
	end process;

	rounded <= std_logic_vector(din + rounder);

	roundwcalc: process (coefid)
	begin
		case coefid is
			when 0 =>	roundw <= o"2";	-- w=5
			when 1 =>	roundw <= o"2";	-- w=5
			when 2 =>	roundw <= o"1";	-- w=6
			when 3 =>	roundw <= o"2";	-- w=5
			when 4 =>	roundw <= o"2";	-- w=5
			when 5 =>	roundw <= o"2";	-- w=5
			when 6 =>	roundw <= o"2";	-- w=5
			when 7 =>	roundw <= o"2";	-- w=5
			when 8 =>	roundw <= o"2";	-- w=5
			when 9 =>	roundw <= o"2";	-- w=5
			when 10 =>	roundw <= o"3";	-- w=4
			when 11 =>	roundw <= o"3";	-- w=4
			when 12 =>	roundw <= o"2";	-- w=5
			when 13 =>	roundw <= o"3";	-- w=4
			when 14 =>	roundw <= o"3";	-- w=4
			when 15 =>	roundw <= o"4";	-- w=3
			when 16 =>	roundw <= o"3";	-- w=4
			when 17 =>	roundw <= o"3";	-- w=4
			when 18 =>	roundw <= o"3";	-- w=4
			when 19 =>	roundw <= o"3";	-- w=4
			when 20 =>	roundw <= o"3";	-- w=4
			when 21 =>	roundw <= o"4";	-- w=3
			when 22 =>	roundw <= o"4";	-- w=3
			when 23 =>	roundw <= o"3";	-- w=4
			when 24 =>	roundw <= o"3";	-- w=4
			when 25 =>	roundw <= o"4";	-- w=3
			when 26 =>	roundw <= o"4";	-- w=3
			when 27 =>	roundw <= o"4";	-- w=3
			when 28 =>	roundw <= o"5";	-- w=2
			when 29 =>	roundw <= o"4";	-- w=3
			when 30 =>	roundw <= o"4";	-- w=3
			when 31 =>	roundw <= o"4";	-- w=3
			when 32 =>	roundw <= o"4";	-- w=3
			when 33 =>	roundw <= o"4";	-- w=3
			when 34 =>	roundw <= o"4";	-- w=3
			when 35 =>	roundw <= o"4";	-- w=3
			when 36 =>	roundw <= o"5";	-- w=2
			when 37 =>	roundw <= o"4";	-- w=3
			when 38 =>	roundw <= o"4";	-- w=3
			when 39 =>	roundw <= o"4";	-- w=3
			when 40 =>	roundw <= o"5";	-- w=2
			when 41 =>	roundw <= o"5";	-- w=2
			when 42 =>	roundw <= o"5";	-- w=2
			when 43 =>	roundw <= o"6";	-- w=1
			when 44 =>	roundw <= o"6";	-- w=1
			when 45 =>	roundw <= o"5";	-- w=2
			when 46 =>	roundw <= o"5";	-- w=2
			when 47 =>	roundw <= o"5";	-- w=2
			when 48 =>	roundw <= o"5";	-- w=2
			when 49 =>	roundw <= o"6";	-- w=1
			when 50 =>	roundw <= o"6";	-- w=1
			when 51 =>	roundw <= o"6";	-- w=1
			when 52 =>	roundw <= o"6";	-- w=1
			when 53 =>	roundw <= o"6";	-- w=1
			when 55 =>	roundw <= o"6";	-- w=1
			when 56 =>	roundw <= o"6";	-- w=1
			when 57 =>	roundw <= o"6";	-- w=1
			when others =>	roundw <= o"6";
		end case;
	end process;

	quantizator: process (clk)
	begin
		if rising_edge(clk) then
			dout <= (others => 'X');
			doute <= '0';
			if dine = '1' then
				case coefid is
					when 0 =>
						collectbuf(0 to 4) <= rounded(7 downto 3);
					when 1 =>
						doute <= '1';
						dout <= collectbuf(0 to 4) & rounded(7 downto 5);
						collectbuf(0 to 1) <= rounded(4 downto 3);
					when 2 =>
						doute <= '1';
						dout <= collectbuf(0 to 1) & rounded(7 downto 2);
					when 3 =>
						collectbuf(0 to 4) <= rounded(7 downto 3);
					when 4 =>
						doute <= '1';
						dout <= collectbuf(0 to 4) & rounded(7 downto 5);
						collectbuf(0 to 1) <= rounded(4 downto 3);
					when 5 =>
						collectbuf(2 to 6) <= rounded(7 downto 3);
					when 6 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
						collectbuf(0 to 3) <= rounded(6 downto 3);
					when 7 =>
						doute <= '1';
						dout <= collectbuf(0 to 3) & rounded(7 downto 4);
						collectbuf(0 to 0) <= rounded(3 downto 3);
					when 8 =>
						collectbuf(1 to 5) <= rounded(7 downto 3);
					when 9 =>
						doute <= '1';
						dout <= collectbuf(0 to 5) & rounded(7 downto 6);
						collectbuf(0 to 2) <= rounded(5 downto 3);
					when 10 =>
						collectbuf(3 to 6) <= rounded(7 downto 4);
					when 11 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
						collectbuf(0 to 2) <= rounded(6 downto 4);
					when 12 =>
						doute <= '1';
						dout <= collectbuf(0 to 2) & rounded(7 downto 3);
					when 13 =>
						collectbuf(0 to 3) <= rounded(7 downto 4);
					when 14 =>
						doute <= '1';
						dout <= collectbuf(0 to 3) & rounded(7 downto 4);
					when 15 =>
						collectbuf(0 to 2) <= rounded(7 downto 5);
					when 16 =>
						collectbuf(3 to 6) <= rounded(7 downto 4);
					when 17 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
						collectbuf(0 to 2) <= rounded(6 downto 4);
					when 18 =>
						collectbuf(3 to 6) <= rounded(7 downto 4);
					when 19 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
						collectbuf(0 to 2) <= rounded(6 downto 4);
					when 20 =>
						collectbuf(3 to 6) <= rounded(7 downto 4);
					when 21 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
						collectbuf(0 to 1) <= rounded(6 downto 5);
					when 22 =>
						collectbuf(2 to 4) <= rounded(7 downto 5);
					when 23 =>
						doute <= '1';
						dout <= collectbuf(0 to 4) & rounded(7 downto 5);
						collectbuf(0 to 0) <= rounded(4 downto 4);
					when 24 =>
						collectbuf(1 to 4) <= rounded(7 downto 4);
					when 25 =>
						doute <= '1';
						dout <= collectbuf(0 to 4) & rounded(7 downto 5);
					when 26 =>
						collectbuf(0 to 2) <= rounded(7 downto 5);
					when 27 =>
						collectbuf(3 to 5) <= rounded(7 downto 5);
					when 28 =>
						doute <= '1';
						dout <= collectbuf(0 to 5) & rounded(7 downto 6);
					when 29 =>
						collectbuf(0 to 2) <= rounded(7 downto 5);
					when 30 =>
						collectbuf(3 to 5) <= rounded(7 downto 5);
					when 31 =>
						doute <= '1';
						dout <= collectbuf(0 to 5) & rounded(7 downto 6);
						collectbuf(0 to 0) <= rounded(5 downto 5);
					when 32 =>
						collectbuf(1 to 3) <= rounded(7 downto 5);
					when 33 =>
						collectbuf(4 to 6) <= rounded(7 downto 5);
					when 34 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
						collectbuf(0 to 1) <= rounded(6 downto 5);
					when 35 =>
						collectbuf(2 to 4) <= rounded(7 downto 5);
					when 36 =>
						collectbuf(5 to 6) <= rounded(7 downto 6);
					when 37 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
						collectbuf(0 to 1) <= rounded(6 downto 5);
					when 38 =>
						collectbuf(2 to 4) <= rounded(7 downto 5);
					when 39 =>
						doute <= '1';
						dout <= collectbuf(0 to 4) & rounded(7 downto 5);
					when 40 =>
						collectbuf(0 to 1) <= rounded(7 downto 6);
					when 41 =>
						collectbuf(2 to 3) <= rounded(7 downto 6);
					when 42 =>
						collectbuf(4 to 5) <= rounded(7 downto 6);
					when 43 =>
						collectbuf(6 to 6) <= rounded(7 downto 7);
					when 44 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
					when 45 =>
						collectbuf(0 to 1) <= rounded(7 downto 6);
					when 46 =>
						collectbuf(2 to 3) <= rounded(7 downto 6);
					when 47 =>
						collectbuf(4 to 5) <= rounded(7 downto 6);
					when 48 =>
						doute <= '1';
						dout <= collectbuf(0 to 5) & rounded(7 downto 6);
					when 49 =>
						collectbuf(0 to 0) <= rounded(7 downto 7);
					when 50 =>
						collectbuf(1 to 1) <= rounded(7 downto 7);
					when 51 =>
						collectbuf(2 to 2) <= rounded(7 downto 7);
					when 52 =>
						collectbuf(3 to 3) <= rounded(7 downto 7);
					when 53 =>
						collectbuf(4 to 4) <= rounded(7 downto 7);
					when 55 =>
						collectbuf(5 to 5) <= rounded(7 downto 7);
					when 56 =>
						collectbuf(6 to 6) <= rounded(7 downto 7);
					when 57 =>
						doute <= '1';
						dout <= collectbuf(0 to 6) & rounded(7 downto 7);
					when others => null;
				end case;

				coefid <= (coefid + 1) mod 64;
			end if;

			if reset = '1' then
				coefid <= 0;
				doute <= '0';
				dout <= (others => 'X');
			end if;
		end if;
	end process;

end arh;
