library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.txt_util.all;

entity test is
end test;

architecture arh of test is

	constant W : integer := 352;
	constant H : integer := 288;

	signal clk : std_logic := '0';
	signal cam_clk : std_logic;
	signal cam_y, cam_uv : std_logic_vector(7 downto 0);
	signal cam_href, cam_vsync : std_logic;

	signal sync, reset : std_logic := '0';

	signal buf_out : std_logic_vector(7 downto 0);
	signal buf_oute, buf_outex : std_logic;
	signal dct_out : signed(7 downto 0);
	signal dct_busy, dct_oute : std_logic;
	signal quant_out : std_logic_vector(7 downto 0);
	signal quant_oute : std_logic;
	signal zip_out : std_logic_vector(7 downto 0);
	signal zip_oute, zip_idle : std_logic;
	signal aligned_out : std_logic_vector(7 downto 0);
	signal aligned_oute : std_logic;

	signal pic_done : boolean;
	signal dout, dout2 : std_logic_vector(31 downto 0);
	signal doute : std_logic;

	signal ffempty, ffrde : std_logic;
	signal fslfull : std_logic := '0';
	signal capt_pause : std_logic;

begin

	clk50: process
	begin
		for i in 1 to 4 loop
			clk <= not clk;
			wait for 10 ns;
		end loop;
		reset <= '1';
		for i in 1 to 2 loop
			clk <= not clk;
			wait for 10 ns;
		end loop;
		reset <= '0';
		while not pic_done loop
			clk <= not clk;
			wait for 10 ns;
		end loop;
		print("clk50 end");
		wait;
	end process;

	generator: entity work.ppmload
		generic map (filename => "lenna.txt.ppm")
		port map (cam_clk, cam_y, cam_uv, cam_href, cam_vsync, pic_done);

	doublebuf: entity work.capturer
		port map (clk, reset, cam_clk, cam_href, cam_vsync, cam_y, cam_uv, capt_pause, buf_out, buf_oute, sync);

	capt_pause <= dct_busy or fslfull;
	--buf_oute <= buf_outex and not fslfull;

	dct2d: entity work.dct2d
		port map (clk, sync, buf_out, buf_oute, dct_out, dct_oute, dct_busy);

	quantizator: entity work.quantizer
		port map (clk, sync, dct_oute, dct_out, quant_oute, quant_out);

	zipper: entity work.compressor
		port map (clk, sync, quant_out, quant_oute, zip_out, zip_oute, zip_idle);

	aligner: entity work.alignator
		generic map (maxraws => 352*288/64*(22+11+11))
		port map (clk, sync, zip_out, zip_oute, zip_idle, quant_oute, aligned_out, aligned_oute);

	merger: entity work.merger
		port map (clk, sync, aligned_out, aligned_oute, dout, doute);

	ofifo: entity work.fifo
		generic map (len => 16, width => 32)
		port map (clk, '0', dout, doute, dout2, ffrde, ffempty);

	ffrde <= not ffempty and not fslfull;

	process
	begin
		for i in 0 to 14440 loop
			wait until rising_edge(clk);
		end loop;
		fslfull <= '1';
		for i in 1 to 128 loop
			wait until rising_edge(clk);
		end loop;
		fslfull <= '0';
		wait;
	end process;

	process (clk)
		variable cnt, num : integer := 0;
	begin
		if rising_edge(clk) then
			if ffrde = '1' then
				print("num: " & str(dout2));
--			if doute = '1' then
				--if num < 2048 then
--					print("num: " & str(dout));
				--end if;
				--num := num + 1;
			end if;
		end if;
	end process;

end arh;
