library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity zzbuf is
	generic ( width : integer := 8);
	port (	clk	: in	std_logic;
		reset	: in	std_logic;
		we	: in	std_logic;
		rowin	: in	std_logic_vector(0 to width*8-1);
		dout	: out	signed(width-1 downto 0);
		doute	: out	std_logic
		);
end zzbuf;

architecture arh of zzbuf is

	type buf2d is array(0 to 7) of std_logic_vector(0 to width*8-1);
	signal buff : buf2d;

	signal addrw, eloutx, elouty : integer range 0 to 7;
	signal elout : std_logic_vector(5 downto 0) := o"00";
	signal doutrow : std_logic_vector(0 to width*8-1);
	signal estart, start : std_logic := '0';

begin

	-- ispuna po retcima
	process (clk)
	begin
		if rising_edge(clk) then
			if we = '1' then
				buff(addrw) <= rowin;
				addrw <= (addrw + 1) mod 8;
			end if;

			if reset = '1' then
				addrw <= 0;
			end if;
		end if;
	end process;

	estart <= '1' when we = '1' and addrw = 0 else '0';
	eloutx <= to_integer(unsigned(elout(5 downto 3)));	-- transponirano!
	elouty <= to_integer(unsigned(elout(2 downto 0)));
	doutrow <= buff(elouty);
	doute <= '1' when elout /= o"00" or start = '1' else '0';
	dout <= signed(doutrow(eloutx*8 to eloutx*8+7));

	delay: process (clk)
	begin
		if rising_edge(clk) then
			if reset = '1' then
				start <= '0';
			else
				start <= estart;
			end if;
		end if;
	end process;

	-- zig-zag cim dobije prvi redak
	process (clk)
	begin
		if rising_edge(clk) then
			case elout is
				when o"10" => elout <= o"01";
				when o"01" => elout <= o"02";
				when o"02" => elout <= o"11";
				when o"11" => elout <= o"20";
				when o"20" => elout <= o"30";
				when o"30" => elout <= o"21";
				when o"21" => elout <= o"12";
				when o"12" => elout <= o"03";
				when o"03" => elout <= o"04";
				when o"04" => elout <= o"13";
				when o"13" => elout <= o"22";
				when o"22" => elout <= o"31";
				when o"31" => elout <= o"40";
				when o"40" => elout <= o"50";
				when o"50" => elout <= o"41";
				when o"41" => elout <= o"32";
				when o"32" => elout <= o"23";
				when o"23" => elout <= o"14";
				when o"14" => elout <= o"05";
				when o"05" => elout <= o"06";
				when o"06" => elout <= o"15";
				when o"15" => elout <= o"24";
				when o"24" => elout <= o"33";
				when o"33" => elout <= o"42";
				when o"42" => elout <= o"51";
				when o"51" => elout <= o"60";
				when o"60" => elout <= o"70";
				when o"70" => elout <= o"61";
				when o"61" => elout <= o"52";
				when o"52" => elout <= o"43";
				when o"43" => elout <= o"34";
				when o"34" => elout <= o"25";
				when o"25" => elout <= o"16";
				when o"16" => elout <= o"07";
				when o"07" => elout <= o"17";
				when o"17" => elout <= o"26";
				when o"26" => elout <= o"35";
				when o"35" => elout <= o"44";
				when o"44" => elout <= o"53";
				when o"53" => elout <= o"62";
				when o"62" => elout <= o"71";
				when o"71" => elout <= o"72";
				when o"72" => elout <= o"63";
				when o"63" => elout <= o"54";
				when o"54" => elout <= o"45";
				when o"45" => elout <= o"36";
				when o"36" => elout <= o"27";
				when o"27" => elout <= o"37";
				when o"37" => elout <= o"46";
				when o"46" => elout <= o"55";
				when o"55" => elout <= o"64";
				when o"64" => elout <= o"73";
				when o"73" => elout <= o"74";
				when o"74" => elout <= o"65";
				when o"65" => elout <= o"56";
				when o"56" => elout <= o"47";
				when o"47" => elout <= o"57";
				when o"57" => elout <= o"66";
				when o"66" => elout <= o"75";
				when o"75" => elout <= o"76";
				when o"76" => elout <= o"67";
				when o"67" => elout <= o"77";
				when o"77" => elout <= o"00";
				when others =>  -- o"00"
					if start = '1' then
						elout <= o"10"; -- krenuo
					else
						elout <= o"00"; -- ceka start
					end if;
			end case;

			if reset = '1' then
				elout <= o"00";
			end if;
		end if;
	end process;

end arh;
